#!/bin/ash


version=3.1.4-festo0
boot=redboot.bin
bootfpga=o3_boot08.fpga
unprot=flash_unprotect.ko
fpgaR1=o83I.fpga
fpgaR2=o83I_ibis.fpga
fpgaR3=o83I_wvga.fpga
kernel=zImage
rfs=rfs_cpu3.gz
rcs=rcS

print_help ()
{
    echo -e "\n\nUpdate camera firmware to version $version"
    echo -e "\t --with-boot|-b         ... also update bootloader and fpga"
    echo -e "\t --with-configblocks|-c ... factory-default configuration blocks"
    echo -e "\t --auto-detect|-a       ... force camera type autodetection"
    echo -e "\t --help|-h              ... this help message\n\n"
}

while [ -n "$1" ];
do
  case "$1" in
    --with-configblocks|-c)
      WITH_CONFIG=1
      ;;
    --auto-detect|-a)
      AUTO_DETECT=1
      ;;
    --with-boot|-b)
      WITH_BOOT=1
      ;;
    --help|-h)
      print_help;
      exit 1;
      ;;
    *)
    echo "unknown parameter: $1"
  esac
shift
done;

print_help;

METHOD=
if [ -z "$AUTO_DETECT" ]; then
  while [ "$METHOD" != "0" -a "$METHOD" != "1" ];
  do
    echo -e "\n\nchoose prefered camera type selection"
    echo "[0] auto detect camera type" 
    echo "[1] manual select camera type"
    echo -n "? " 
    read METHOD
  done; 
else
  METHOD=0
fi;

HW_STRING=`cat /proc/config/hardware`

if [ "$METHOD" = "0" ]; then
  C_FPGA=`expr substr $HW_STRING 16 1`
  case "$C_FPGA" in
    1)
      fpga=$fpgaR1
      BUS_EXTENSION=
    ;;
    2)
      fpga=$fpgaR2
      BUS_EXTENSION=
    ;;
    3)
      fpga=$fpgaR3
      BUS_EXTENSION="-WB"
    ;;
    *)
    echo "autodetection of FPGA design failed. aborting."
    exit 2;
  esac

  C_CASE=`expr substr $HW_STRING 11 1`
  if [ "$C_CASE" != "C" -a "$C_CASE" != "I" ]; then
    echo "autodetection of FPGA design failed. aborting."
    exit 2;
  fi

  C_COLOR=`expr substr $HW_STRING 17 1`
  if [ "$C_COLOR" != "B" -a "$C_COLOR" != "C" ]; then
    echo "autodetection of FPGA design failed. aborting."
    exit 2;
  fi

  cfg0=config-blocks/config-block-0-${C_CASE}-R${C_FPGA}${C_COLOR}${BUS_EXTENSION}
  cfg1=config-blocks/config-block-1-R$C_FPGA

  echo -e "\n\nFound Sensor R$C_FPGA, using $fpga as FPGA design.\n\n"

else
  CAM_TYPE=-1
  while [ $CAM_TYPE -lt 0 -o $CAM_TYPE -gt 11 ];
  do
    echo -e "\n\nManually select Camera Type"
    echo "[0]  SBOC-C-R1B" 
    echo "[1]  SBOC-C-R1C" 
    echo "[2]  SBOI-C-R1B" 
    echo "[3]  SBOI-C-R1C" 
    echo
    echo "[4]  SBOC-C-R2B" 
    echo "[5]  SBOC-C-R2C" 
    echo "[6]  SBOC-C-R2B-A" 
    echo "[7]  SBOC-C-R2C-A" 
    echo
    echo "[8]  SBOC-C-R3B-WB" 
    echo "[9]  SBOC-C-R3C-WB" 
    echo "[10] SBOC-I-R3B-WB" 
    echo "[11] SBOC-I-R3C-WB" 
    echo -n "? " 
    read CAM_TYPE
  done; 

  case "$CAM_TYPE" in
    0)
      fpga=$fpgaR1
      cfg0=config-blocks/config-block-0-C-R1B
      cfg1=config-blocks/config-block-1-R1
    ;;
    1)
      fpga=$fpgaR1
      cfg0=config-blocks/config-block-0-C-R1C
      cfg1=config-blocks/config-block-1-R1
    ;;
    2)
      fpga=$fpgaR1
      cfg0=config-blocks/config-block-0-I-R1B
      cfg1=config-blocks/config-block-1-R1
    ;;
    3)
      fpga=$fpgaR1
      cfg0=config-blocks/config-block-0-I-R1C
      cfg1=config-blocks/config-block-1-R1
    ;;
    4)
      fpga=$fpgaR2
      cfg0=config-blocks/config-block-0-C-R2B
      cfg1=config-blocks/config-block-1-R2
    ;;
    5)
      fpga=$fpgaR2
      cfg0=config-blocks/config-block-0-C-R2C
      cfg1=config-blocks/config-block-1-R2
    ;;
    6)
      fpga=$fpgaR2
      cfg0=config-blocks/config-block-0-C-R2B-A
      cfg1=config-blocks/config-block-1-R2
    ;;
    7)
      fpga=$fpgaR2
      cfg0=config-blocks/config-block-0-C-R2C-A
      cfg1=config-blocks/config-block-1-R2
    ;;
    8)
      fpga=$fpgaR3
      cfg0=config-blocks/config-block-0-C-R3B-WB
      cfg1=config-blocks/config-block-1-R3
    ;;
    9)
      fpga=$fpgaR3
      cfg0=config-blocks/config-block-0-C-R3C-WB
      cfg1=config-blocks/config-block-1-R3
    ;;
    10)
      fpga=$fpgaR3
      cfg0=config-blocks/config-block-0-I-R3B-WB
      cfg1=config-blocks/config-block-1-R3
    ;;
    11)
      fpga=$fpgaR3
      cfg0=config-blocks/config-block-0-I-R3C-WB
      cfg1=config-blocks/config-block-1-R3
    ;;
    *)
    echo "error in case. aborting."
    exit 2;
  esac
fi

do_flash ()
{
    echo Flashing $1
    flash_unlock $2
    flash_eraseall $2
    echo "programming ..."
    cat $1 > $2
    echo done
    echo
}

test_file ()
{
  if test ! -r $1; then echo $1 is not available; exit; fi
  sum=`./md5sum $1 | cut -b -32`
  if test $2 != $sum
  then 
    echo "checksum error in $1"
    exit
  fi
}

# FPGA design, Kernel, RootFileSystem
do_flash $fpga /dev/mtd2
do_flash $kernel /dev/mtd3
do_flash $rfs /dev/mtd4

# Bootloader + Boot-FPGA
if [ -n "$WITH_BOOT" ]; then
insmod flash_unprotect.ko
  do_flash $boot /dev/mtd0
  do_flash $bootfpga /dev/mtd1
  rmmod flash_unprotect
fi

# Config Blocks 0,1
if [ -n "$WITH_CONFIG" ]; then
  do_flash $cfg0 /dev/mtd7
  do_flash $cfg1 /dev/mtd8
fi

# /ffx
#rm -Rf /ffx/*
if test -e /ffx/rcS; then 
  echo renaming /ffx/rcS to /ffx/rcS.old
  mv /ffx/rcS /ffx/rcS.old
fi
cp $rcs /ffx
tar xvzf ffx.tar.gz -C /ffx

./fconfig_set_firmware $version
./fconfig_set_defaultpw
sync

echo
echo '----------------------------------------- !!!'
echo 'your password for user root was reset to  !!!'
echo 'the factory default: "Festo"              !!!'
echo '----------------------------------------- !!!'
echo
echo reboot the system, please
echo
 
